from pathlib import Path
from typing import cast

from truss.remote.baseten.remote import BasetenRemote
from truss.remote.remote_factory import RemoteFactory
from truss_train.deployment import create_training_job_from_file


def push(config: Path, remote: str = "baseten"):
    """Create or update a training project and create a training job.

    This function performs the following operations:
    1. Creates or updates a training project
    2. Creates a training job within the training project

    Args:
        config: Path to the Python file that defines the training project and job.
        remote: The remote provider to use. Defaults to "baseten".

    Returns:
        dict: A dictionary containing the created training project and job:
            {
                "training_project": TrainingProject,
                "training_job": TrainingJob,
            }

        For detailed definitions of TrainingProject and TrainingJob, see the API docs:
        https://docs.baseten.co/reference/training-api/get-training-job
    """
    remote_provider: BasetenRemote = cast(
        BasetenRemote, RemoteFactory.create(remote=remote)
    )
    return create_training_job_from_file(remote_provider, config)



"""
The `Model` class is an interface between the ML model that you're packaging and the model
server that you're running it on.

The main method to implement here is:
* `predict`: runs every time the model server is called. Include any logic for model
  inference and return the model output.

See https://truss.baseten.co/quickstart for more.
"""

import truss_chains as baseten


class Model(baseten.ModelBase):
    # Configure resources for your model here.
    remote_config: baseten.RemoteConfig = baseten.RemoteConfig(name="{{ MODEL_NAME }}")

    def __init__(
        self, context: baseten.DeploymentContext = baseten.depends_context()
    ) -> None:
        # Access secrets via optional `context` variable.
        # Load model here and assign to self._model.
        self._model = None

    def predict(self, input_field: int) -> int:
        # Run model inference here
        return input_field



from abc import ABC, abstractmethod
from pathlib import Path
from typing import Optional

from truss.util.docker import Docker
from truss.util.path import given_or_temporary_dir


class ImageBuilder(ABC):
    def build_image(
        self,
        build_dir: Optional[Path] = None,
        tag: Optional[str] = None,
        labels: Optional[dict] = None,
        cache: bool = True,
        network: Optional[str] = None,
    ):
        """Build image.

        Arguments:
            build_dir(Path): Directory to use for building the docker image. If None
                             then a temporary directory is used.
            tag(str): A tag to assign to the docker image.
        """

        with given_or_temporary_dir(build_dir) as build_dir_path:
            self.prepare_image_build_dir(build_dir_path)
            return Docker.client().build(
                str(build_dir_path),
                labels=labels if labels else {},
                tags=tag or self.default_tag,
                cache=cache,
                network=network,
                load=True,  # We pass load=True so that `build` returns an `Image` object
            )

    @property
    @abstractmethod
    def default_tag(self):
        pass

    @abstractmethod
    def prepare_image_build_dir(self, build_dir: Optional[Path] = None):
        pass

    def docker_build_command(self, build_dir) -> str:
        return f"docker build {build_dir} -t {self.default_tag}"
